package sudoku;

import java.util.Random;
import java.awt.*;

/** quelques algorithmes pour résoudre ou créer des SU DOKU
  * <br><i> several algorithms to solve or create SU DOKU </i> */

public class Algo{

   /** la position sur le diagramme 
     * <br><i>the initial position on the board</i> */

   public int cells[][];
   
   /** effectue la copie d'une position 
     * <br><i>make a copy of a position</i> */
   public static Cell[][] clone(Cell[][] cells){
      int i,j,k,n=cells.length;
      Cell[][] c=new Cell[n][n];
      for(i=0;i<n;i++)
      for(j=0;j<n;j++) {
         c[i][j]=new Cell(1);
         for(k=0;k<cells[i][j].list.size();k++){
            Candidat cand=cells[i][j].get(k);
            Candidat newc=new Candidat(cand.value);
            newc.color=cand.color;
            newc.position=cand.position;
            c[i][j].add(newc);
         }
      }
      return c;
   }
   
   /** un algorithme de remplissage d'un nouveau SU DOKU ... Pas très efficace : on doit pouvoir faire mieux
     * <br><i> a method to create a SU DOKU ... not very efficient, one should be able to improve it</i> */
   public static boolean build(Cell[][] cells){
      int i,j,k,n=cells.length;Cell test[][]=clone(cells);String value;
      for(i=0;i<n;i++)for(j=0;j<n;j++) {test[i][j].clear();cells[i][j].clear();}
      for(i=0;i<n;i++)for(j=0;j<n;j++)for(k=0;k<n;k++)  test[i][j].add(Position.symbols[k]);
      int ns=100;
      Random r=new Random();
      while(ns != 1){
         i=r.nextInt(n);
         j=r.nextInt(n);
         Cell c=test[i][j];
         if(c.list.size() > 1){
            value=c.get(r.nextInt(c.list.size())).value;
            System.out.println("testing "+value+" at i="+i+" j="+j);
            Cell[][] copie=clone(test);
            Position.fixValue(i,j,value,copie);
            ns=fastSolver(copie,0);
            System.out.println("ns ="+ns);
            if(ns >= 1) {
               Position.fixValue(i,j,value,test);
               cells[i][j].add(value);
            }
         }
      }
      for(i=0;i<n;i++)for(j=0;j<n;j++){
         if(cells[i][j].list.size() == 1){
            test=clone(cells);
            value=test[i][j].get(0).value;
            test[i][j].clear();
            ns=fastSolver(test,0);
            if(ns == 1) {
               System.out.println("redundant value "+value+" i="+i+" j="+j);
               cells[i][j].clear();
            }
         }
      } 
      Position.print(cells);
      return true;
   }
   
   /** résolution d'un SU DOKU 
     * <br><i>solving a SU DOKU</i> */
   public static boolean solve(Cell[][] cells,int level){
      int i,j,ii,jj,k,n=cells.length;
      Cell[][] copie;
      for(i=0;i<n;i++)
      for(j=0;j<n;j++) 
         if(cells[i][j].list.size() != 1) Position.fillCandidats(cells[i][j],cells);      
      for(i=0;i<n;i++)for(j=0;j<n;j++)
         if(cells[i][j].list.size() == 0) return false;
      for(i=0;i<n;i++)for(j=0;j<n;j++)
         if(cells[i][j].list.size() > 1) {
            for(k=cells[i][j].list.size()-1;k>=0;k--){
               copie=clone(cells);
               copie[i][j].clear();
               Candidat c=new Candidat(cells[i][j].get(k).value);
               c.setColor(Color.BLUE);
               copie[i][j].add(c);
               if(!solve(copie,level++)) {
                  cells[i][j].delete(k);
                  if(k == 0) return false;
               }else{
                  for(ii=0;ii<n;ii++)for(jj=0;jj<n;jj++) cells[ii][jj]=copie[ii][jj];
                  return true;
               }   
            }
         }
      return true;
   }

   /** une tentative d'amélioration de l'algorithme de "solve" avec calcul du nombre de solutions 
     * <br><i>an attempt to improve "solve" and computation of the number of solutions </i>*/
   public static int fastSolver(Cell[][] cells,int level){
      int i,j,imin=-1,jmin=-1,k,n=cells.length;
      Cell[][] copie,solution=null;
      int min=10;
      if(level == 0) Position.fill(cells);
      for(i=0;i<n;i++)for(j=0;j<n;j++){
         int s=cells[i][j].list.size();
         if(s > 1 && s < min) {
            imin=i;
            jmin=j;
            min=s;
         }else if(s == 0) return 0;
      }
      if(min == 10) return 1;
      int nsol=0,nsol1=0;
      for(k=min-1;k>=0;k--){
         copie=clone(cells);
         String value=cells[imin][jmin].get(k).value;
         Position.fixValue(imin,jmin,value,copie);
         nsol1=fastSolver(copie,level++);
         cells[imin][jmin].delete(k);
         if(nsol1 > 0) solution=copie;
         nsol=nsol+nsol1;
         if(nsol > 1) break;
      }
      if(nsol > 0) for(i=0;i<n;i++)for(j=0;j<n;j++) {
         cells[i][j].clear();
         cells[i][j].add(solution[i][j].get(0));
      }
      return nsol;
   }
   
   /** programme de test 
     * <br><i>simple test programme </i> */
   static public void main(String[] a){
      Position p=new Position(9,30);
      p.setInitialPosition("A2=2 A4=7 A5=4 A9=3 B1=6 B3=8 B6=1 B9=7 C3=3 C6=8 C8=4 C9=5 D4=8 D8=6 E3=4 E7=9 F2=7 F6=5 G1=9 G2=8 G4=5 G7=4 H1=4 H4=9 H7=8 H9=1 I1=3 I5=8 I6=6 I8=5");
      System.out.println("Position initiale");
      Position.print(p.cells);
      Position.fill(p.cells); // fill candidats
      System.out.println("number of solutions "+fastSolver(p.cells,0));
      System.out.println("Position finale");
      Position.print(p.cells);
      build(p.cells);
   }
}         
