package sudoku;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

/** <br>une appliquette pour afficher des problèmes rangés par catégorie dans une série de fichiers 
  * <br><i>an applet to propose problems organised by categories in several files</i>
  * <br> */

public class Base extends Applet implements ItemListener,ActionListener{

/** le panneau d'affichage
  * <br><i>the drawing board </i>*/
   public Board board;
   
/** l'index du problème actuel
  * <br><i>the index of the current problem in the current file</i>*/
   public int igame;
/** la position initiale du problème courant
  * <br><i>the inital position of the current problem </i>*/
   public String currentgame;
/** problème précédent
  * <br><i>previous problem </i>*/
   public Button prev;
/** problème suivant
  * <br><i>next problem </i>*/
   public Button next;
/** les fichiers contenant chacun un ensemble de problèmes
  * <br><i>the files, each of them containing a set of problems </i>*/
   public FileList bases;
/** les problèmes du fichier courant
  * <br><i>the problems of the current file </i>*/
   public Vector games=new Vector();
/** le nom du fichier courant
  * <br><i>the current file name </i>*/
   public String basefile;

/** la méthode d'initialisation de l'applet
  * <i>the method to initialize the applet</i>*/
   public void init(){
      int i;
      RF.Init(getParameter("lang"),getClass());
      setLayout(new BorderLayout());
      int size=getWidth()/15;
      board=new Board(9,size);
      setFont(new Font("Helvetica",Font.PLAIN,14));
      
      bases=new FileList(this);
      bases.addItemListener(this);
      basefile=bases.getFile(0);
      loadGames(basefile);
      
      prev=new Button(RF.get("button.init"));
      next=new Button(RF.get("button.next"));
      Panel p=new Panel();
      
      Info info=new Info(this);
      
      p.add(bases);
      p.add(prev);
      p.add(next);
      p.add(info);
      
      add("North",p);
      add("Center",board);
      prev.addActionListener(this);
      next.addActionListener(this);
      currentgame=(String)games.elementAt(0);
      board.position.setPosition(currentgame);
      board.repaintAll();
   }
   
/** <br>lecture d'un fichier contenant des problèmes
  * <br><i>reading a file containing problems</i>*/
   public void loadGames(String file){
     System.out.println("file "+file);
     games.removeAllElements();
     int i=0;
     try{
       URL u=new URL(getDocumentBase(),file);
       BufferedReader r=new BufferedReader(new InputStreamReader(u.openStream()));
       while(true){
         try{
           String s=r.readLine();
           if(s == null) break;
           System.out.println(s);
           games.addElement(s);
         }catch(Exception e){break;}
       }
     }
     catch(Exception e){
       System.out.println("Impossible to load the file "+file+" i="+i);
       System.out.println(e.toString());
     }
   }
   
  /** <br>change le fichier de problem
    * <br><i>change the active problem file</i> */
  public void itemStateChanged(ItemEvent e){
    String f=bases.getFile();
    if(! f.equals(basefile)) {
       basefile=f;
       loadGames(f);
       igame=0;
       board.position.setPosition((String)games.elementAt(igame));
       board.repaintAll();
    }
  }

  /**<br> selectionne un problème ou une partie dans le fichier actif
      <br><i>selecting a problem in the current file</i>*/
  public void actionPerformed(ActionEvent e){
    Button b=(Button)e.getSource();
    if(b == next){
      igame++;
      if(igame >= games.size()) igame=0;
    }
    else if(b == prev){
      igame--;
      if(igame < 0) igame=games.size()-1;
    }
    currentgame=(String)games.elementAt(igame);
    board.position.setPosition(currentgame);
    board.repaintAll();
  }

}      
