package sudoku;
   
import java.awt.*;
import java.awt.image.*;
import java.util.*;
import java.io.*;
import javax.imageio.*;

/** affichage d'un diagramme de SU DOKU
  * <br><i> drawing a SU DOKU board</i> */

public class Board extends Panel{

   /** la position du diagramme 
     * <br><i> the position on the board</i>*/
   public Position position;
   /** les quatre zones rectangulaires représentant le bord extérieur du diagramme 
     * <br><i> the four reactangular areas composing the border of the board</i>*/
   public Area up,down,left,right;
   /** la numérotation des colonnes 
     * <br><i> the column numbering</i>*/
   public Symbols hsym;
   /** la numérotation des lignes 
     * <br><i> the row numbering</i>*/
   public Symbols vsym;
   /** la taille d'une cellule
     * <br><i> the cell size</i>*/
   public int size;
   /** la taille du SU DOKU (9x9 par défaut)
     * <br><i> the SU DOKU size (9x9 by default)</i>*/
   public int n=9;
   /** l'épaisseur du cadre
     * <br><i>the frame thickness</i>*/
   public int border=2;
   /** sauvegarde graphique du diagramme
     * <br><i>graphical saving of the board</i>*/
   public Image img;
   
   /** contruire un diagramme dont on fixe les dimensions 
     * <br><i>building a board at the right dimension</i>*/
   public Board(int n,int s){
   
      Cell     cell;Junction junc;Corner   corn;
      int i; char c; String string;
      
      size=s;
      
      if(size < 20) border=1; else border=2;
      
      setLayout(null);
      up   =new Area(Color.BLACK,(n+1)*(size+size/2),border);
      down =new Area(Color.BLACK,(n+1)*(size+size/2),border);
      right=new Area(Color.BLACK,border,(n+1)*(size+size/2));
      left =new Area(Color.BLACK,border,(n+1)*(size+size/2));
      
      string="";
      for(i=0;i<n;i++) {c=(char)('A'+i);string=string+c;}
      hsym =new Symbols(string,size/2+n*(size+size/2),size);
      
      string="";
      for(i=0;i<n;i++) {c=(char)('1'+i);string=string+c;}
      vsym =new Symbols(string,size,(n+1)*(size+size/2));
      
      position=new Position(n,s);
      
      setPosition(position);
      
      resize(s);
   }
      
   /** changer la taille des cellules du diagramme
     * <br><i>changing the cell size</i>*/
   public void resize(int s){
      int i,j;
      size=s;
      
      up.setSize(2*border+(n+1)*(size+size/2),border);
      up.setLocation(0,0);
      
      down.setSize(2*border+(n+1)*(size+size/2),border);
      down.setLocation(0,border+(n+1)*(size+size/2));
      
      left.setSize(border,2*border+(n+1)*(size+size/2));
      left.setLocation(0,0);
      
      right.setSize(border,2*border+(n+1)*(size+size/2));
      right.setLocation(border+(n+1)*(size+size/2),0);

      vsym.setSize(size,(n+1)*(size+size/2));
      vsym.setLocation(border,border);

      hsym.setSize(size/2+n*(size+size/2),size);
      hsym.setLocation(border+size,border+size/2+n*(size+size/2));
      
      setSize(2*border+(n+1)*(size+size/2),2*border+(n+1)*(size+size/2));
      
      for(i=0; i< n;i++)for(j=0; j< n;j++) {
         position.cells[i][j].setSize(size,size);
         position.cells[i][j].setLocation(border+size+size/2+i*(size+size/2),border+size/2+j*(size+size/2));
      }
      for(i=0; i< n+1;i++) for(j=0; j< n;j++){
         position.hjuns[i][j].setSize(size/2,size);
         position.hjuns[i][j].setLocation(border+size+i*(size+size/2),border+size/2+j*(size+size/2));
      }
      for(i=0; i< n;i++) for(j=0; j< n+1;j++){
         position.vjuns[i][j].setSize(size,size/2);
         position.vjuns[i][j].setLocation(border+size+size/2+i*(size+size/2),border+j*(size+size/2));
      }
      for(i=0; i< n+1;i++) for(j=0; j< n+1;j++){
         position.corns[i][j].setSize(size/2,size/2);
         position.corns[i][j].setLocation(border+size+i*(size+size/2),border+j*(size+size/2));
      }
   }
   
   /** afficher une nouvelle position
     * <br><i>drawing a new position</i>*/
   public void setPosition(Position p){
      int i,j;
      removeAll();
      add(up);add(down);add(left);add(right);
      add(hsym); add(vsym);
      position=p;
      for(i=0; i< n  ;i++) for(j=0; j< n  ;j++) add(position.cells[i][j]);
      for(i=0; i< n+1;i++) for(j=0; j< n  ;j++) add(position.hjuns[i][j]);
      for(i=0; i< n  ;i++) for(j=0; j< n+1;j++) add(position.vjuns[i][j]);
      for(i=0; i< n+1;i++) for(j=0; j< n+1;j++) add(position.corns[i][j]);
      resize(size);
      repaint();
   }
   
   /** retracer tous les éléments composant un diagramme
     * <br><i>redrawing all components composing the board</i>*/
   public void repaintAll(){
      for(int j=0;j<getComponentCount();j++) getComponent(j).repaint();
   }
   
   /** changer la couleur de fond
     * <br><i>changing the background color</i>*/
   public void changeBackground(Color oldcolor,Color newcolor){
      for(int j=0;j<getComponentCount();j++) {
         Zone z=(Zone)getComponent(j);
         z.background=newcolor;
         Color b=z.getBackground();
         if(b.equals(oldcolor)){z.setBackground(newcolor);z.back=newcolor;z.repaint();}
      }
   }
      
  /** sélectionner la ligne contenant une zone particulière
    * <br><i>selecting the row containing a particular zone</i>*/
   public void selectRow(Zone z){
      int i,j,k;
      if(z instanceof Cell){
         Cell c=(Cell)z;
         for(i=0;i<n;i++) for(j=0;j<n;j++) if(c.equals(position.cells[i][j])){
            for(k=0;k<n  ;k++) position.cells[k][j].select();
            for(k=0;k<n+1;k++) position.hjuns[k][j].select();
            return;
         }
      }else if(z instanceof Junction){
         Junction c=(Junction)z;
         for(i=0;i<n+1;i++) for(j=0;j<n;j++) if(c.equals(position.hjuns[i][j])){
            for(k=0;k<n  ;k++) position.cells[k][j].select();
            for(k=0;k<n+1;k++) position.hjuns[k][j].select();
            return;   
         }
         for(i=0;i<n;i++) for(j=0;j<n+1;j++) if(c.equals(position.vjuns[i][j])){
            for(k=0;k<n  ;k++) position.vjuns[k][j].select();
            for(k=0;k<n+1;k++) position.corns[k][j].select();
            return;   
         }
      }else if(z instanceof Corner){
         Corner c=(Corner)z;
         for(i=0;i<n+1;i++) for(j=0;j<n+1;j++) if(c.equals(position.corns[i][j])){
            for(k=0;k<n  ;k++) position.vjuns[k][j].select();
            for(k=0;k<n+1;k++) position.corns[k][j].select();
            return;   
         }
      }
      return;
   }
   
  /** sélectionner la colonne contenant une zone particulière
    * <br><i>selecting the column containing a particular zone</i>*/
   public void selectColumn(Zone z){
      int i,j,k;
      if(z instanceof Cell){
         Cell c=(Cell)z;
         for(i=0;i<n;i++) for(j=0;j<n;j++) if(c.equals(position.cells[i][j])){
            for(k=0;k<n  ;k++) position.cells[i][k].select();
            for(k=0;k<n+1;k++) position.vjuns[i][k].select();
            return;
         }
      }else if(z instanceof Junction){
         Junction c=(Junction)z;
         for(i=0;i<n;i++) for(j=0;j<n+1;j++) if(c.equals(position.vjuns[i][j])){
            for(k=0;k<n  ;k++) position.cells[i][k].select();
            for(k=0;k<n+1;k++) position.vjuns[i][k].select();
            return;   
         }
         for(i=0;i<n+1;i++) for(j=0;j<n;j++) if(c.equals(position.hjuns[i][j])){
            for(k=0;k<n  ;k++) position.hjuns[i][k].select();
            for(k=0;k<n+1;k++) position.corns[i][k].select();
            return;   
         }
      }else if(z instanceof Corner){
         Corner c=(Corner)z;
         for(i=0;i<n+1;i++) for(j=0;j<n+1;j++) if(c.equals(position.corns[i][j])){
            for(k=0;k<n  ;k++) position.hjuns[i][k].select();
            for(k=0;k<n+1;k++) position.corns[i][k].select();
            return;   
         }
      }
      return;
   }
   
  /** sélectionner la section contenant une zone particulière
    * <br><i>selecting the section containing a particular zone</i>*/
   public void selectSection(Zone z){
      int i,j;
      Point p=getSection(z);
      if(p==null) return;
      for(i=p.x;i<p.x+3;i++) for(j=p.y;j<p.y+3;j++) position.cells[i][j].select();
      for(i=p.x;i<p.x+4;i++) for(j=p.y;j<p.y+3;j++) position.hjuns[i][j].select();
      for(i=p.x;i<p.x+3;i++) for(j=p.y;j<p.y+4;j++) position.vjuns[i][j].select();
      for(i=p.x;i<p.x+4;i++) for(j=p.y;j<p.y+4;j++) position.corns[i][j].select();
   }
   
  /** sélectionner une zone particulière
    * <br><i>selecting a particular zone</i>*/
   public void selectZone(Zone z){z.select();}
   public void unselectZone(Zone z){z.unselect();}
   
  /** obtenir les coodonnées de la section contenant une zone particulière
    * <br><i>getting the coordinates of a section containing a particular zone</i>*/
   public Point getSection(Zone z){
      int i,j;
      for(i=0;i< n;i++) for(j=0;j< n;j++) if(z.equals((Zone)position.cells[i][j])) {return new Point((i/3)*3,(j/3)*3);}     
      for(i=0;i< n;i++) for(j=0;j< n;j++) if(z.equals((Zone)position.hjuns[i][j])) {return new Point((i/3)*3,(j/3)*3);}     
      for(i=0;i< n;i++) for(j=0;j< n;j++) if(z.equals((Zone)position.vjuns[i][j])) {return new Point((i/3)*3,(j/3)*3);}     
      for(i=0;i< n;i++) for(j=0;j< n;j++) if(z.equals((Zone)position.corns[i][j])) {return new Point((i/3)*3,(j/3)*3);}
      return null;
   }    

  /** vider le diagramme
    * <br><i>emptying the board</i>*/
   public void emptyBoard(){
      position.emptyPosition();
      repaint();
   }
   
  /** construire l'image équivalente au diagramme
    * <br><i>building up the image representing the board</i>*/
   public BufferedImage getBufferedImage(){
      BufferedImage img=new BufferedImage(2*border+(n+1)*(size+size/2),2*border+(n+1)*(size+size/2),BufferedImage.TYPE_INT_RGB);
      Graphics g=img.getGraphics();
      g.setColor(getBackground());
      g.fillRect(0,0,getWidth(),getHeight());
      for(int k=0;k<getComponentCount();k++){
         Zone z=(Zone)getComponent(k);
         Point p=z.getLocation();
         g.drawImage(z.getImage(),p.x,p.y,z.getWidth(),z.getHeight(),this);
      }
      return img;
   }
   
  /** sauver l'image du diagramme dans un fichier PNG dont on fournit le nom
    * <br><i>saving the board image into a PNG file defined by its name</i>*/
   public void saveImage(String filename){
      saveImage(new File(filename));
   }
   
  /** sauver l'image du diagramme dans un fichier PNG
    * <br><i>saving the board image into a PNG file</i>*/
   public void saveImage(File file){
      try{
         BufferedImage img=getBufferedImage();
         ImageIO.write(img,"png",file);
      }catch(IOException e){System.out.println("Impossible to create the file "+file.toString());}
   }
       
  /** programme de test
    * <br><i>simple test programme</i>*/
   public static void main(String[] a){
      int size=50;
      if(a.length > 0) size=Integer.parseInt(a[0]);
      Frame f=new Frame();
      Board board=new Board(9,size);
      RF.Init("fr",board.getClass());
      Position position=new Position(9,size);
      position.setPosition("A1=3n=,A5=4n=,C6=2n=,I7=8n=,C6=5n= C3=8r+ F5=9b-  C4=7g=,F4=8g=");
      board.setPosition(position);
      position.setValue("9",5,3);
      f.add(board);
      f.pack();
      f.show();
   }
}
