package sudoku;

import java.awt.*;

  /** gestion des candidats
    * <br><i> handling candidates ... </i> */

public class Candidat extends Object{


   /** chaîne de caractères représentant le candidat
     * <br><i> string reprenting the candidate</i> */
   public String value;
   /** un petit déplacement par rapport à la position normale. ceci concerne les marques placées sur des jonctions.
     * <br><i> a small shif near the mormal position. It concerns marks located on junctions.</i> */
   public int position;
   /** pour entourer le candidat.
     * <br><i> to draw a circle oround the candidate</i> */
   public boolean circle;
   /** un drapeau indiquant que le candidat est sélectionné
     * <br><i> a flag to indicate the selection of the candidate</i> */
   public boolean selected;
   /** un drapeau indiquant que le candidat est barré
     * <br><i> a flag to strike the candidate</i> */
   public boolean barre;
   /** la couleur du candidat
     * <br><i> the candidate color</i> */
   public Color color;

  /** création d'un candidat avec les paramètres par défaut
    * <br><i>candidate creator with default parameters</i> */
   
   public Candidat(String v){
      value=v;
      position=0;
      selected=false;
      circle=false;
      barre=false;
   }
   
  /** récupérer la valeur du candidat (son symbole associé)
    * <br><i>getting the candidate value (its associated symbol)</i> */
   public String getValue(){return value;}
  /** changer la valeur du candidat
    * <br><i>changing the candidate value</i> */
   public void setValue(String v){value=v;}
  /** changer la couleur du candidat
    * <br><i>changing the candidate color</i> */
   public void setColor(Color c){color=c;}
  /** changer la drapeau rayant le candidat
    * <br><i>changing the strike flag</i> */
   public void changeBarre(){if(barre) barre=false; else barre=true;}
  /** changer légèrement la position du candidat
    * <br><i>slightly modifying the candidate position</i> */
   public void setPosition(int p){if(p < -1 || p > 1) return;position=p;}
  /** récupérer le petit déplacement du candidat
    * <br><i>getting the small position shift</i> */
   public int getPosition(){return position;}
  /** sélectionner le candidat
    * <br><i>selecting the candidate</i> */
   public void select(){selected=true;}
  /** désélectionner le candidat
    * <br><i>unselecting the candidate</i> */
   public void unselect(){selected=false;}
  /** active le tracé d'un cercle autour du candidat
    * <br><i>activate the drawing of a circle around the candidate</i> */
   public void setCircle(){circle=true;}
  /** désactive le tracé d'un cercle autour du candidat
    * <br><i>deactivate the drawing of a circle around the candidate</i> */
   public void unsetCircle(){circle=false;}
   
  /** dessine le candidat
    * <br><i>draws the candidate</i> */
   public void paint(Graphics g,int i, int j){
      int x1=0;int x2=0;int y1=0;int y2=0;
      FontMetrics fm=g.getFontMetrics();
      if(selected) g.setColor(Color.RED); else g.setColor(Color.BLUE);
      int width=fm.stringWidth(value);     
      int height=fm.getAscent()*8/10;
      if(color == null){
         if(selected) g.setColor(Color.RED); else g.setColor(Color.BLUE);
      }else{
         g.setColor(color);
      }
      g.drawString(value,i-width*5/9,j+height/2);
      if(barre){
         g.setColor(Color.BLACK);
         g.drawString("X",i-width*5/9,j+height/2);
      }
   }
   
  /** dessine le candidat
    * <br><i>draws the candidate</i> */
   public void paint(Graphics g,int p,int direction,int size){
      int x1=0;int x2=0;int y1=0;int y2=0;
      FontMetrics fm=g.getFontMetrics();
      System.out.println("candidat "+value+" position "+p);
      System.out.println("direction "+direction+" size "+size);
      if(selected) g.setColor(Color.RED); else g.setColor(Color.BLUE);
      if(direction == 1){
         x1=size/12*(position+1); x2=x1+size/6;
         y1=p*size/3; y2=(p+1)*size/3;
      }else if(direction == 2){
         x1=p*size/3; x2=(p+1)*size/3;
         y1=size/12*(position+1); y2=y1+size/6;
      }else if(direction == 0){ // case of a cell
         if(p == 0) {
            x1=0; x2=size/2; y1=size/2; y2=size;
         }else if(p == 1){
            x1=size/2; x2=size; y1=size/2; y2=size;
         }else if(p == 2){
            x1=0; x2=size/2; y1=0; y2=size/2;
         }else if(p == 3){
            x1=size/2; x2=size; y1=0; y2=size/2;
         }
      }else{ // case of a corner
         x1=0; x2=size-1;y1=0;y2=size-1;
      }
      int width=fm.stringWidth(value);     
      int height=fm.getAscent()*9/10;
      int i=x1+(x2-x1-width)/2;
      int j=y2-(y2-y1-height)/2;
      g.drawString(value,i,j); 
   }
}
