package sudoku;

import java.awt.*;
import java.util.*;

/** définition d'une position complète avec marquage éventuel
  * <br><i>definition of a full position taking into account marks</i> */
public class Position{

/** les cellules interactives principales du diagramme
  * <br><i>main sensitive zones of the board</i> */
   public Cell     cells[][];
/** les junctions horizontales entre deux cellules d'une même colonne
  * <br><i>the horizontal junctions between two cells of a same column</i> */
   public Junction hjuns[][];
/** les junctions verticales entre deux cellules d'une même ligne
  * <br><i>the vertical junctions between two cells of a same row</i> */
   public Junction vjuns[][];
/** les coins généralement à l'intersection de 4 cellules
  * <br><i>the corner zones, usually located at the cross section of four cells</i> */
   public Corner   corns[][];
/** la dimension du diagramme (habituellement 9)
  * <br><i>the board dimension (usually 9)</i> */
   public int n;
/** les symboles utilisés pour représenter les candidats
  * <br><i>the symobols used to represent candidates</i> */
   public static final String[] symbols={"1","2","3","4","5","6","7","8","9","A","B","C","D","E","F","G"};

/** résolution des candidats dans chaque cellule
  * <br><i>solving candidates in each cell</i> */
   public static void fill(Cell cells[][]){
      int i,j,n=cells.length;
      for(i=0;i<n;i++)
      for(j=0;j<n;j++) 
         if(cells[i][j].list.size() != 1) fillCandidats(cells[i][j],cells);
   }
   
/** constructeur
  * <br><i>constructor</i> */
   public Position(int n,int size){
      int i,j;
      Cell cell;Junction junc; Corner corn;
      this.n=n;
      cells=new Cell[n][n];
      hjuns=new Junction[n+1][n];
      vjuns=new Junction[n][n+1];
      corns=new Corner[n+1][n+1];
      for(i=0; i< n;i++){
      for(j=0; j< n;j++){
         cell = new Cell(size);
         cells[i][j]=cell;
      }}
      boolean f;
      for(i=0; i< n+1;i++){
      for(j=0; j< n;j++){
         f=false;
         if(i % 3 == 0) f=true;
         junc = new Junction(size/2,size,f);
         hjuns[i][j]=junc;
      }}
      for(i=0; i< n;i++){
      for(j=0; j< n+1;j++){
         f=false;
         if(j % 3 == 0) f=true;
         junc = new Junction(size,size/2,f);
         vjuns[i][j]=junc;
      }}
      int type;
      for(i=0; i< n+1;i++){
      for(j=0; j< n+1;j++){
         type=Corner.C;
         if(i % 3 == 0 && j % 3 == 0) {
            type=Corner.CC;
            if(i == 0) type=Corner.LC;
            if(i == n) type=Corner.RC;
            if(j == 0) type=Corner.CU;
            if(j == n) type=Corner.CL;
            if(i == 0 && j == 0) type=Corner.UL;
            if(i == 0 && j == n) type=Corner.LL;
            if(i == n && j == 0) type=Corner.UR;
            if(i == n && j == n) type=Corner.LR;
         }
         if(i % 3 == 0 && j % 3 != 0) {
            type = Corner.Cc;
            if(i == 0) type=Corner.Lc;
            if(i == n) type=Corner.Rc;
         }
         if(i % 3 != 0 && j % 3 == 0) {
            type = Corner.cC;
            if(j == 0) type=Corner.cU;
            if(j == n) type=Corner.cL;
         }
         corn = new Corner(size/2,type);
         corns[i][j]=corn;
      }}
   }
   
/** initialise un diagramme à partir d'une chaîne décrivant une position
  * <br><i>initializing a position from a string describing it</i> */
   
   public void setPosition(String p){
      StringTokenizer st=new StringTokenizer(p," ");
      emptyPosition();
      while(st.hasMoreTokens()) play(st.nextToken());
   }
   
/** obtenir la couleur correspondant à une caractère spécifique
  * <br><i>color corresponding to a particular char</i> */
   public static Color getColor(char c){
      switch(c){
         case 'r' : return Color.RED;
         case 'b' : return Color.BLUE;
         case 'g' : return Color.GREEN;
         default  : return Color.BLACK;
      }
   }   
   
/** vider une position
  * <br><i>emptying a position</i> */
   public void emptyPosition(){
      int i,j;
      for (i=0; i< n; i++) for (j=0; j< n; j++) cells[i][j].clear();
      for (i=0; i<=n; i++) for (j=0; j< n; j++) hjuns[i][j].clear();
      for (i=0; i< n; i++) for (j=0; j<=n; j++) vjuns[i][j].clear();
      for (i=0; i<=n; i++) for (j=0; j<=n; j++) corns[i][j].clear();
   }
   
/** obtenir le symbole d'une colonne
  * <br><i>getting a column symbol</i> */
   public static char getColumn(int i){
      switch(i){
         case 0 : return 'A';
         case 1 : return 'B';
         case 2 : return 'C';
         case 3 : return 'D';
         case 4 : return 'E';
         case 5 : return 'F';
         case 6 : return 'G';
         case 7 : return 'H';
         case 8 : return 'I';
         default : return ' ';
      }
   }
   
/** localiser une cellule appartenant à une position
  * <br><i>locating a cell belonging to a position</i> */
   static public Point getCellPosition(Cell c,Cell cells[][]){
      int i,j,n=cells.length;
      Point p=new Point(-1,-1);
      for(i=0; i< n;i++) for(j=0;j<n;j++) if(c.equals(cells[i][j])) {p.x=i;p.y=j;return p;}
      return p;
   }
   
/** résoudre les candidats d'une cellule particulière
  * <br><i>solving candidates in a particular cell</i> */
   public static void fillCandidats(Cell c,Cell cells[][]){
      int i,j,k,l,n;
      n=cells.length;
      Point p=getCellPosition(c,cells);
      i=p.x; j=p.y;
      c.clear();
      for(k=0;k<n;k++) c.add(new Candidat(symbols[k]));
      for(k=0; k<n ; k++) 
         if(k != i){
            Cell cc=cells[k][j];
            if(cc.list.size() == 1) c.delete(cc.get(0));
         }
      for(k=0; k<n ; k++) 
         if(k != j){
            Cell cc=cells[i][k];
            if(cc.list.size() == 1) c.delete(cc.get(0));
         }
      int isect=(i/3)*3;
      int jsect=(j/3)*3;
      for(k=isect;k<isect+3;k++)
      for(l=jsect;l<jsect+3;l++)
         if(k != i && l != j){
            Cell cc=cells[k][l];
            if(cc.list.size() == 1) c.delete(cc.get(0));
         }
   }
   
/** placer une candidat définitif dans une cellule
  * <br><i>fixing the final unique candidate of a cell</i> */
   public static void fixValue(int i,int j,String value,Cell cells[][]){
      int k,l,nc,n=cells.length;
      cells[i][j].clear();
      for(k=0;k<n;k++) if(i!=k) {
         nc=cells[k][j].list.size();
         cells[k][j].delete(value);
         if(cells[k][j].list.size() == 1 && nc != 1) fixValue(k,j,cells[k][j].get(0).value,cells);
      }  
      for(k=0;k<n;k++) if(j!=k){ 
         nc=cells[i][k].list.size();
         cells[i][k].delete(value);
         if(cells[i][k].list.size() == 1 && nc != 1) fixValue(i,k,cells[i][k].get(0).value,cells);
      }
      int isect=(i/3)*3;
      int jsect=(j/3)*3;
      for(k=isect;k<isect+3;k++)for(l=jsect;l<jsect+3;l++) if(k != i && l != j) {
         nc=cells[k][l].list.size();
         cells[k][l].delete(value);
         if(cells[k][l].list.size() == 1 && nc != 1) fixValue(k,l,cells[k][l].get(0).value,cells);
      }
      cells[i][j].add(value);
   }
      
/** indice de ligne d'une cellule particulière
  * <br><i>row index of a particular cell</i> */
   public int getRowIndex(Cell c){
      int i,j;
      for(i=0; i< n;i++) for(j=0;j<n;j++) if(c.equals(cells[i][j])) return j;
      return -1;
   }
    
/** indice de colonne d'une cellule particulière
  * <br><i>column index of a particular cell</i> */
   public int getColumnIndex(Cell c){
      int i,j;
      for(i=0; i< n;i++) for(j=0;j<n;j++) if(c.equals(cells[i][j])) return i;
      return -1;
   }
   
/** sélectionne toute une ligne
  * <br><i>selecting a whole row</i> */
   public void selectRow(int j){
      int i;
      for(i=0;i< n;i++) cells[i][j].select();
      for(i=0;i<=n;i++) hjuns[i][j].select();
   }
   
/** sélectionne toute une colonne
  * <br><i>selecting a whole column</i> */
   public void selectColumn(int i){
      int j;
      for(j=0;j< n;j++) cells[i][j].select();
      for(j=0;j<=n;j++) vjuns[i][j].select();
   }
   
/** indice associé à un symbole
  * <br><i>symbol index</i> */
   public static int index(char c){
      switch(c){
         case '1' : case 'a' : case 'A' : return 0;
         case '2' : case 'b' : case 'B' : return 1;
         case '3' : case 'c' : case 'C' : return 2;
         case '4' : case 'd' : case 'D' : return 3;
         case '5' : case 'e' : case 'E' : return 4;
         case '6' : case 'f' : case 'F' : return 5;
         case '7' : case 'g' : case 'G' : return 6;
         case '8' : case 'h' : case 'H' : return 7;
         case '9' : case 'i' : case 'I' : return 8;
         case 'j' : case 'J' : return 9;
         case 'k' : case 'K' : return 10;
      }
      return 0;
   }
   
/** enregistrement des candidats dans une chaîne;
  * <br><i>registering candidates within a string</i> */
   static public String saveCandidates(Zone[][] zones,int ni,int nj,String prefix){
      int i,j,k;
      String s="";
      for(i=0;i<ni;i++) for(j=0;j<nj;j++) for(k=0;k<zones[i][j].list.size();k++){
         Candidat c=zones[i][j].get(k);
         s=s+prefix+getColumn(i)+getRow(j)+"="+c.value;
         if(c.color==Color.RED)
            s=s+'r';
         else if(c.color==Color.BLUE)
            s=s+'b'; 
         else if(c.color==Color.GREEN)
            s=s+'g'; 
         switch(c.position){
            case  1 : s=s+'+'; break;
            case -1 : s=s+'-'; break;
         }
         s=s+" ";
      }
      return s;
   }
   
/** chaîne de caractères équivalente à la position complète
  * <br><i>building the string equivalent to the full position</i> */
   public String saveFullPosition(){
      String s;
      s=saveCandidates(cells,n  ,n  ,"" )
       +saveCandidates(hjuns,n+1,n  ,"h")
       +saveCandidates(vjuns,n  ,n+1,"v")
       +saveCandidates(corns,n+1,n+1,"c");
      return s;
   }
   
/** symbole associé à une ligne particulière
  * <br><i>symbol associated to a particular row</i> */
   public static char getRow(int j){
      switch(j){
         case 0 : return '1';
         case 1 : return '2';
         case 2 : return '3';
         case 3 : return '4';
         case 4 : return '5';
         case 5 : return '6';
         case 6 : return '7';
         case 7 : return '8';
         case 8 : return '9';
         default : return ' ';
      }
   }
   
/** initialiser une position à partir d'une chaîne de caractères
  * <br><i>initializing the position from a string</i> */
   public void setInitialPosition(String p){
      StringTokenizer st=new StringTokenizer(p," ");
      emptyPosition();
      while(st.hasMoreTokens()){
         String term=st.nextToken();
         int i=index(term.charAt(0));
         int j=index(term.charAt(1));
         String value=term.substring(3);
         Candidat c=new Candidat(value);
         c.setColor(Color.BLACK);
         cells[i][j].add(c);
      }
   }
   
/** prendre en compte un seul candidat nouveau représenté par un texte court
  * <br><i>setting up only one candidate represented by a short string</i> */
   public void play(String s){
      System.out.println("move : "+s);
      int k;
      String move=s;
      int point=s.indexOf('.');
      if(point >= 0) move=move.substring(point+1);
      Zone[][] zone=cells;
      if(move.startsWith("c")){
         zone=corns; move=move.substring(1);
      }else if(move.startsWith("h")){
         zone=hjuns; move=move.substring(1);
      }else if(move.startsWith("v")){
         zone=vjuns; move=move.substring(1);
      }
      int i=index(move.charAt(0));
      int j=index(move.charAt(1));
      String value=null;
      if(move.length() > 3) value=move.substring(3,4);
      System.out.println("value = "+value+" move="+move+" i="+i+" j="+j);
      Color color=Color.BLACK;
      if(move.length() > 4)color=Position.getColor(move.charAt(4));
      int pos=0;
      if(move.length() > 5){
         if(move.charAt(5) == '+') pos=1;
         if(move.charAt(5) == '-') pos=-1;
      }  
      Candidat cand=null;        
      if(value != null) {
         cand=new Candidat(value);
         cand.setColor(color);
         cand.setPosition(pos);
      }
      switch(move.charAt(2)){
         case '?' : zone[i][j].add(cand);break;
         case '=' : zone[i][j].clear(); zone[i][j].add(cand);break;
         case 'x' : k=zone[i][j].getIndex(value); if(k >= 0) zone[i][j].delete(k);break; 
         case 'X' : zone[i][j].clear();break;
      }
   }           
   
/** fixer le candidat final d'une cellule
  * <br><i>setting the final candidate of a cell</i> */
   public void setValue(String v,int column,int row){
      Cell cell=cells[column][row];
      cell.clear();
      cell.add(new Candidat(v));
   }
           
/** ajouter un candidat dans une cellule
  * <br><i>adding a candidate in a cell</i> */
   public void addValue(String v,int column,int row){
      Cell cell=cells[column][row];
      cell.add(new Candidat(v));
   }
   
/** imprimer la position principale, c'est à dire les cellules contenant un seul candidat
  * <br><i>printing the main position, i.e. the cells containing a unique candidate</i> */
   public static void print(Cell[][] cells){
      int i,j,n=cells.length;
      for(j=0;j< n;j++){
         String s="";
         for(i=0;i<n;i++){
            Cell c=cells[i][j];
            if(c.list.size() == 1) s=s+" "+c.get(0).value; else s=s+"  ";
         }
         System.out.println(s);
      }
   }
   
/** imprimer la position complète
  * <br><i>printing the full position</i> */
   public static void printFull(Cell[][] cells){
      int i,j,k,l,n=cells.length;
      for(j=0;j< n;j++){
         String s="";
         for(i=0;i<n;i++){
            Cell c=cells[i][j];
            s=s+" ";
            for(l=0;l < c.list.size(); l++) s=s+c.get(l).value;
            for(l=c.list.size(); l< n;l++) s=s+" ";
         }
         System.out.println(s);
      }
   }

}
