package sudoku;

import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.Color;

/**
 * Une classe qui gère les langues. Par exemple, les applets de
 * SU DOKU sont disponible en anglais et en français grâce à des
 * fichiers de langues config.en et config.fr
 *
 * Class managing the resource files in order to authorize
 * any language and to get any user dependent parameters
 *
 * Various files can be loaded :
 * - config.xx : a resource file associated to a language xx
 * and located in the directory containing the class RF
 * - yyyyyyy : a file associated to the application calling RF
 * and located in the user's home directory
 * 
 */

public class RF {

  /** les propriétés gérées par la classe <br><i>the list of properties managed by the class<i> */
  public static Properties r=null;
  public static Properties ruser=null;
  public static Class rootclass;

  /** lecture d'un fichier de resources assicié à une langue<br><i>read the resource file associated to a given language</i> */

  public static void Init(String language,Class root){
    rootclass=root;
    r=new Properties();
    String filename="config";
    if(language != null) filename=filename+"."+language;
    System.out.println("filename = "+filename);
    System.out.println("resource = "+root.getResource(filename));
    
    try{
      InputStream is=root.getResource(filename).openStream();
      r.load(is);
    }
    catch(Exception e){System.out.println(e.toString());}
  }

  /** lecture d'un fichier de ressources situé dans le répertoire principal de l'utilisateur
    * <br><i>read a resource file located in the user's home directory</i> */
  
  public static void load(String userfile){
    if(ruser == null) ruser=new Properties();
    try{
      File f=new File(System.getProperty("user.home"),userfile);
      ruser.load(new FileInputStream(f));
    }
    catch(Exception e){}
  }

  /** récupérer une propriété associée à une clé
    * <br><i>gets a property (string) associated to a key. A default result is provided too</i> */
  
  public static String get(String l,String def){
    String result=null;
    if(ruser  != null) result=ruser.getProperty(l);
    if(result == null) result=r.getProperty(l);
    if(result== null) result=def;
    return result;
  }

  /** récupérer une propriété associée à une clé (par défault, la clé est renournée)
    * <br><i>gets a property (string) associated to a key. By default the key is returned</i> */
  
  public static String get(String l){
    return get(l,l);
  }

  /** stocker une propriété de type "chaîne" associée à une clé
    * <br><i>sets a property (string) associated to a key</i>.*/
  
  public static void set(String l,String v){
    if(ruser==null) ruser=new Properties();
    ruser.put(l,v);
  }
  
  /** stocker une propriété de type "entier" associée à une clé
    * <br><i>sets a property (int) associated to a key</i<.*/
  
  public static void set(String l,int v){
    if(ruser==null) ruser=new Properties();
    ruser.put(l,""+v);
  }
  
  /** stocker une propriété de type "réel" associée à une clé
    * <br><i>sets a property (float) associated to a key</i<.*/
  
  public static void set(String l,float v){
    if(ruser==null) ruser=new Properties();
    ruser.put(l,""+v);
  }
  
  /** stocker une propriété de type "réel double" associée à une clé
    * <br><i>sets a property (double) associated to a key</i<.*/
  
  public static void set(String l,double v){
    if(ruser==null) ruser=new Properties();
    ruser.put(l,""+v);
  }

  /** récupérer une propriété de type "chaîne"
    * <br><i>get a string property. By default, null is returned</i> */
  
  public static String getString(String l){
    return get(l);
  }

  /** récupérer une propriété de type "chaîne" en précisant la valeur par défaut
    * <br><i>get a string property fixing the default value</i> */
  
  public static String getString(String l, String def){
    return get(l,def);
  }

  /** récupère une popriété de type "entier" (0 par défaut)
    * <br><i>get an integer property. By default, 0 is returned</i>. */
  
  public static int getInt(String l){
    try{return Integer.parseInt(get(l));}
    catch(Exception e){return 0;}
  }
  
  /** récupère une popriété de type "entier" en fixant une valeur par défaut
    * <br><i>get an integer property fixing a default value</i>. */

  public static int getInt(String l,int def){
    try{return Integer.parseInt(get(l));}
    catch(Exception e){return def;}
  }

  /** récupère une propriété de type "réel" (défaut 0.)
    * <br><i>get a float property. By default, 0. is returned.</i> */

  public static float getFloat(String l){
    try{return Double.valueOf(get(l)).floatValue();}
    catch(Exception e){return (float)0.;}
  }

  /** récupère une popriété de type "réel" en précisant la valeur par défaut
    * <br><i> get a float property precising a default value</i>*/

  public static float getFloat(String l, float def){
    try{return Double.valueOf(get(l)).floatValue();}
    catch(Exception e){return def;}
  }
  

  /** récupère une popriété de type "double" (défaut 0.)
    * <br><i>get a double property. By default, 0. is returned.</i> */

  public static double getDouble(String l){
    try{return Double.valueOf(get(l)).doubleValue();}
    catch(Exception e){return 0.;}
  }

  /** récupère une popriété de type "double" en précisant la valeur par défaut
    * <br><i> get a double property precising a default value</i>*/

  public static double getDouble(String l, double def){
    try{return Double.valueOf(get(l)).doubleValue();}
    catch(Exception e){return def;}
  }

  /** récupérer une couleur en précisant la couleur par défaut
    * <br><i>get a Color property precising a default value</i>*/

  public static Color getColor(String l, Color def){
    return Color.getColor(l,def);
  }

  /** récupérer une couleur (noir par défaut)
    * <br><i>get a Color property (default : black)</i>*/

  public static Color getColor(String l){
    return Color.getColor(l,Color.BLACK);
  }
  
  /** sauvegarder l'ensemble des propriétés dans un fichier du répertoire principal de l'utilisateur
    * <br><i>saving the current properties in a file located in the user home directory</i> */
  
  public static void save(String userfile){
    try{
      File f=new File(System.getProperty("user.home"),userfile);
      FileOutputStream os=new FileOutputStream(f);
      ruser.store(os,null);
      os.flush();
      os.close();
    }
    catch(Exception e){
      System.out.println("Impossible to create the file "+userfile);
    }
  }
  
  /** nom de répertoire d'un fichier
    * <br><i>get a directory </i>*/

  public static String getUp(String filename){
    File f=new File(filename);
    String s=f.getParent();
    if(s == null)
      return "./";
    else
      return s;
  }
  
  /** obtenir le nom d'un sous-repertoire situé dans le répertoire principal de l'utilisateur
    * <br><i>get a directory at the same level as the home directory of the class RF </i>*/

  public static String getHome(String subdir){
    File f=new File(getUp(getHome()),subdir);
    return f.getPath();
  }
  
  /** répertoire principal de l'utilisateur
    * <br><i>get the user root directory </i>*/

  public static String getUserHome(){
    String dir=System.getProperty("user.home");
    return dir;
  }
  
  /** nom du système d'exploitation
    * <br><i>get the operating system </i>*/
  
  public static String getOS(){
    String fs=System.getProperty("file.separator");
    if(fs.equals("/"))
      return "unix";
    else
      return "dos";
  }
  
  /** nom du répertoire contenant la classe RF
    * <br> <i> get the home directory of the class RF </i>*/

  public static String getHome(){
    try{
      String s=RF.getString("sigal.home","");
      if(s.equals("")) {
        s=rootclass.getResource("config").getFile();
        String ss=new File(s).getParent();
        ss= new File(ss).getParent();
        return ss;
      }else{
        File f=new File(s,"java");
        return f.getPath();
      }
    }
    catch(Exception e){System.out.println("impossible to get the home directory");}
    return "";
  }

}
