package sudoku;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.io.*;
import java.net.*;

/** une applet pour rejouer des parties de SU DOKU. Les parties sont enregistrées dans des fichiers "texte" 
  * <br><i> an applet to replay SU DOKU games. Games are recorded in independent text files  </i> */

public class Replay extends Applet implements ActionListener,MoveListener{

/** le diagramme de SU DOKU
  * <br><i> the SU DOKU board </i> */
   public Board board;   
/** une zone de texte pour écrire les commentaires
  * <br><i> A text zone to write comments </i> */
   public TextZone text;   
/** le nombre de parties
  * <br><i> the number of games </i> */
   public int ngame;
/** le numéro de la partie en cours
  * <br><i> the index of the current game </i> */
   public int currentgame;
/** le bouton affichant la 1ère partie
  * <br><i> come back to the first game</i> */
   public Button first;
/** le bouton pour changer de problème
  * <br><i> the button to change the game </i> */
   public Button next;
/** le bouton pour redémarrer la partie courante
  * <br><i> the button to restart the current game </i> */
   public Button firstmove;
/** le bouton pour reprendre le dernier coup joué
  * <br><i> the button to suppress the last move </i> */
   public Button prevmove;
/** le bouton pour avancer dans la partie
  * <br><i> next move </i> */
   public Button nextmove;
/** le bouton voir la position finale
  * <br><i> final position </i> */
   public Button lastmove;
/** le nombre de coups + 1 (= nb de positions enregistrées)
  * <br><i> the number of moves + 1 (=nb of recorded positions </i> */
   public int nmove;
/** le numéro du dernier coup joué
  * <br><i> index of the last move </i> */
   public int currentmove;
/** les coups, le premier (indice 0) étant en fait la position initiale
  * <br><i> the moves, the first one being the initial position </i> */
   public String lang;
/** la langue utilisée pour commenter les parties ("fr" pour le français)
  * <br><i> the language used to comment the game ("en" for English) </i> */
   public Vector moves;
/** la police utilisée
  * <br><i> the font size </i> */
   public int fontsize=12;

   public void init(){
   
      setBackground(new Color(230,230,230));
      lang=getParameter("lang");
      if(lang == null) lang="fr";
      RF.Init(lang,getClass());
      setLayout(new BorderLayout());
      
      Panel left=new Panel();
      left.setLayout(new BorderLayout());
      Panel right=new Panel();
      right.setLayout(new BorderLayout());
      
      add("Center",left); add("East",right);
      
      int size=(getHeight()-50)/15;
      board=new Board(9,size);
      setFont(new Font("Helvetica",Font.PLAIN,14));
      first=new Button(RF.get("button.first"));first.addActionListener(this);
      next=new Button(RF.get("button.next"));next.addActionListener(this);
      
      Panel p=new Panel(); p.setLayout(new GridLayout(1,2));
      p.add(first); p.add(next);
      left.add("North",p);
      left.add("Center",board);
      
      firstmove=new Button("<<");firstmove.addActionListener(this);
      prevmove =new Button("<" ); prevmove.addActionListener(this);
      nextmove =new Button(">" ); nextmove.addActionListener(this);
      lastmove =new Button(">>"); lastmove.addActionListener(this);
      
      p=new Panel();
      p.setLayout(new GridLayout(1,4));
      p.add(firstmove);p.add(prevmove);p.add(nextmove);p.add(lastmove);
      left.add("South",p);
      
      Scrollbar vertical  =new Scrollbar(Scrollbar.VERTICAL,1,getHeight()/fontsize,1,2000);
      vertical.setSize(14,getHeight());
      text =new TextZone(getWidth()-board.getWidth()-vertical.getWidth()-2,getHeight(),"Serif",fontsize);
      text.setScrollbar(vertical);
      
      text.addMoveListener(this);
      
      // right.setSize(getWidth()-board.getWidth(),getHeight());
      right.add("Center",text);right.add("East",vertical);
      //text.setLocation(vertical.getWidth(),0); vertical.setLocation(0,0);
      
      ngame=0;
      while(getParameter(RF.get("game")+ngame) != null) ngame++;
      
      loadGame(currentgame);
    }
    
    public void loadGame(int i){
      currentgame=i;
      if(i >= ngame) currentgame=0;
      moves=Move.readFile(getParameter(RF.get("game")+currentgame)+'.'+lang,this);
      nmove=moves.size();
      currentmove=0;
      text.firstdraw=0;
      setMove(currentmove);
    }
    
    public void actionPerformed(ActionEvent e){
      Button b=(Button)e.getSource();
      if(b == first)
         loadGame(0);
      else if(b == next)
         loadGame(++currentgame);
      else if(b == firstmove)
         setMove(0);
      else if(b == prevmove)
         setMove(--currentmove);
      else if(b == nextmove)
         nextMove();
      else if(b == lastmove)
         setMove(nmove-1);
    }
    
    public void setMove(int i){
      if(i < 0) return;
      if(currentmove > 0) unselectZone(currentmove);
      board.position.setPosition((String)moves.get(0));
      text.setText("",0);
      text.appendParagraph("&bf "+RF.get("text.initial")+" &rm ");
      text.appendString("\n");
      text.appendParagraph((String)moves.get(0));
      text.appendString("\n");
      for(int k=1; k <= i; k++) applyMove(k);
      currentmove=i;
      if(currentmove > 0) selectZone(currentmove);
      board.repaintAll();
      text.showLastLine();
      text.repaint();
    }
    
    public void selectZone(int k){
      String smove=(String)moves.get(k);
      Move.strong(board,smove);
    }
    
    public void unselectZone(int k){
      String smove=(String)moves.get(k);
      Move.normal(board,smove);
    }
    
    public void applyMove(int k){
      String smove=(String)moves.get(k);
      smove=smove.trim();
      String comment="";
      if(smove.length() == 0) {
         text.appendString("\n");
         return;
      }
      int startcomment=smove.indexOf(' ');
      if(startcomment > 0){ 
         comment=smove.substring(startcomment); 
         smove=smove.substring(0,startcomment);
      }
      Move.executeMove(board,smove);
      text.appendParagraph("&bf &blue "+k+". &rm &4 &red "+smove+" &black &0");
      text.appendParagraph(comment);
    }      
    
    public void nextMove(){
      if(currentmove >= nmove-1) return;
      if(currentmove > 0) unselectZone(currentmove);
      currentmove++;
      applyMove(currentmove);
      text.showLastLine();
      text.repaint();
      if(currentmove > 0) selectZone(currentmove);
    }
    
    public void moveSelected(int num){setMove(num);}

}
