package sudoku;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

/** zone de texte colorisée avec plusieurs polices possibles. Ils suffit d'insérer
  * des mots clés particuliers :
  * <ul><li> &rm police normale
  *     <li> &bf police grasse
  *     <li> &it police italique
  *     <li> &red couleur rouge
  *     <li> &green couleur verte
  *     <li> &blue couleur bleue
  *     <li> &black couleur noire
  *     <li> &b espace
  *     <li> &xx positionnement absolu = xx*largeur moyenne d'un caractère
  * </ul>
  * <P><i> colorised text zone with several possible fonts. Special key words have to be inserted :
  * <ul>
  *     <li> &rm normal font </li>
  *     <li> &bf bold </li>
  *     <li> &it italic </li>
  *     <li> &red red color </li>
  *     <li> &green green color </li>
  *     <li> &blue blue color </li>
  *     <li> &black black color </li>
  *     <li> &b space </li>
  *     <li> &xx absolute position = xx * average character width
  * </ul></i>
  */

public class TextZone extends Canvas implements AdjustmentListener,KeyListener,MouseListener{

  /** numéro de la première ligne visible
      <br><i>index of the first row at the top of the text zone</i> */
  public int firstline=1;

  /** texte<br><i>text</i> */
 public  StringBuffer text=new StringBuffer(5000);

  /** éventuelle barre de défilement vertical associée à la zone de texte
      <br><i>vertical scroll bar associated to the text zone</i> */
  public Scrollbar vertical;

  /** nombre de lignes visibles
      <br><i>total number of rows in the text zone</i> */
  public int nrow;

  /** nombre total de lignes
      <br><i>number of visible rows in the text zone</i> */
  public int nline;

  /** dernière ligne visible (par défaut toutes les lignes sont visibles)
      <br><i>the last visible line (by default, all lines are visible)</i>*/
  public int lastvisibleline=-1;

  /** dernière colonne visible (par défaut toutes les colonnes sont visibles)
      <br><i>the last visible column (by default, all columns are visible)</i>*/
  public int lastvisiblecolumn=-1;

  /** largeur de la zone de texte <br><i>text zone width</i> */
  public int width;

  /** marge à gauche <br><i>left margin</i> */
  public int imin=2;

  /** largeur moyenne d'un caractère <br><i>average char width</i> */
  public int charwidth;

  /** hauteur de la zone de texte <br><i>text zone height</i> */
  int height;

  /** taille de la police courante <br><i>current font size</i> */
  public int fontsize;

  /** police normale <br><i>plain font</i> */
  public Font rm;

  /** police grasse <br><i>bold font</i> */
  public Font bf;

  /** police italique <br><i>italic font</i> */
  public Font it;

  /** caratéristiques de la police courante <br><i>current font metrics</i> */
  public FontMetrics fm;

  /** position dans la ligne <br><i>current position in a line</i> */
  private int ipos=2;

  /** chaîne racine pour analyse <br><i>root string for internal analysis</i> */
  public String rootstring;

  /** position dans la chaîne racine <br><i>current index in the root string</i> */
  public int iroot;

  /** première ligne dessinée <br><i>first line drawn</i> */
  public int firstdraw=0;
  
  /** écouteur de mouvement <i>(move listener)</i> */
  MoveListener ml;

  /** construit une zone de texte <br><i>build a text zone</i>
   *  @param width largeur en pixels <br><i>text width in pixels</i>
   *  @param fontname nom de base des polices <br><i>font name family</i> ("Serif" "SansSerif" "Monospaced" ...)
   *  @param fontsize taille des polices <br><i>font size</i>
   */

  public TextZone(int width,int height,String fontname,int fontsize){

    setSize(width,height);
    
    addKeyListener(this);
    addMouseListener(this);

    setFont(fontname,fontsize);
    fm=getFontMetrics(bf);
    charwidth=fm.charWidth('w');
    this.fontsize=fm.getHeight();
    nrow=(height-this.fontsize/2)/this.fontsize;

    this.width=width;
    this.height=height;
    
  }

  /** change les polices <br><i>change the font family</i>
    * @param fontname nom de base des polices <br><i>family name</i>
    * @param fontsize taille <br><i>size</i>
    */

  public void setFont(String fontname,int fontsize){
    rm=new Font(fontname,Font.PLAIN,fontsize);
    bf=new Font(fontname,Font.BOLD,fontsize);
    it=new Font(fontname,Font.ITALIC,fontsize);
  }

  /** met en service une barre de défilement vertical
      <br><i>introduce a vertical scroll bar</i>*/
  public void setScrollbar(Scrollbar vertical){
    this.vertical=vertical;
    vertical.addAdjustmentListener(this);
    vertical.setValues(firstline,nrow,1,getLineNumber());
  }

  /** construit une s=zone de texte avec des polices par défaut
      <br><i>build a text zone with default fonts</i> */

  public TextZone(int width,int height){
    this(width,height,"Serif",12);
  }
  
  public int getVisible(){return nrow;}

  /** fournit le dernier caractère du texte
      <br><i>provide the last chararcter of the text</i>*/
  public char lastChar(){
    int l=text.length();
    if(l == 0) return '\n';
    return text.charAt(l-1);
  }

  /** initialise le texte écrit dans la zone
      <br><i>initialize the text written in the text zone</i>
   * @param s nouveau texte <br><i>new text</i>
   * @param line numéro de la ligne apparaissant en haut de la fenêtre 
   <br><i>the row at the top of the text zone</i>
   */

  public void setText(String s, int line){
    text.setLength(0);
    ipos=imin;
    appendText(s);
  }

  /** récupérer le texte <br><i>get the text</i> */
  public String getText(){
    return text.toString();
  }

  /** modifie la première ligne (défilement)
      <br><i>change the first line at the top of the zone (scrolling)</i> */
  public void setRow(int line){
    firstline=line;
    if(firstline <=0) firstline=1;
    repaint();
  }

  /** ajoute une ligne <br><i>append a new line</i> */
  public void appendLine(String s){
    if(lastChar() != '\n') text.append('\n');
    text.append(s);
    text.append('\n');
    ipos=imin;
  }

  /** ajoute une chaîne de caractères à la dernière ligne
      <br><i>add a string to the last line</i>*/
  public void appendString(String s){
    text.append(s);
    if(lastChar() == '\n') ipos=imin; else  ipos=ipos+getLength(s);
  }
  
  /** ajoute un texte qui s'ajustera automatiquement à la largeur de la zone
      <br><i>append a text automatically adjusted at the zone width</i>*/
  public void appendText(String ss){
    String tab="&0";
    int len;
    int itab=imin;
    StringTokenizer st=new StringTokenizer(ss," \n\r");
    while(st.hasMoreTokens()){
      String s=st.nextToken();
      if(s.charAt(0) != '&'){
        int isave=ipos; // to avoid a JAVA bug : the variable ipos is reassign somewhere
        len=fm.stringWidth(s);
        if(isave+len > width) {
          text.append('\n'+tab+' ');
          isave=imin;
        }
        len=getLength(s);
        ipos=isave+len;
      }else{
        if     (s.equals("&rm")) fm=getFontMetrics(rm);
        else if(s.equals("&it")) fm=getFontMetrics(it);
        else if(s.equals("&bf")) fm=getFontMetrics(bf);
        else if(s.equals("&b" )) ipos=ipos+charwidth;
        else if(s.equals("&0" )) {tab=s;itab=imin;ipos=itab;}
        else if(s.equals("&1" )) {tab=s;itab=imin+charwidth;ipos=itab;}
        else if(s.equals("&2" )) {tab=s;itab=imin+charwidth*2;ipos=itab;}
        else if(s.equals("&3" )) {tab=s;itab=imin+charwidth*3;ipos=itab;}
        else if(s.equals("&4" )) {tab=s;itab=imin+charwidth*4;ipos=itab;}
        else if(s.equals("&5" )) {tab=s;itab=imin+charwidth*5;ipos=itab;}
        else if(s.equals("&6" )) {tab=s;itab=imin+charwidth*6;ipos=itab;}
        else if(s.equals("&7" )) {tab=s;itab=imin+charwidth*7;ipos=itab;}
        else if(s.equals("&8" )) {tab=s;itab=imin+charwidth*8;ipos=itab;}
        else if(s.equals("&9" )) {tab=s;itab=imin+charwidth*9;ipos=itab;}
        else if(s.equals("&10")) {tab=s;itab=imin+charwidth*10;ipos=itab;}
        else if(s.equals("&11")) {tab=s;itab=imin+charwidth*11;ipos=itab;}
        else if(s.equals("&12")) {tab=s;itab=imin+charwidth*12;ipos=itab;}
        else if(s.equals("&13")) {tab=s;itab=imin+charwidth*13;ipos=itab;}
        else if(s.equals("&14")) {tab=s;itab=imin+charwidth*14;ipos=itab;}
        else if(s.equals("&15")) {tab=s;itab=imin+charwidth*15;ipos=itab;}
      }
      text.append(s);
      text.append(' ');
    }
  }

  /** longueur d'une chaîne suivie d'un espace en pixels
      <br><i>length of a string followed by a space (in pixel)</i> */
  public int getLength(String s){return fm.stringWidth(s+' ');}

  /** ajoute un paragraphe ajusté automatiquement à la largeur de la zone
     <br><i>append a paragraph (adjusted text with end-line characters</i> */

  public void appendParagraph(String s){
    if(lastChar() != '\n') text.append('\n');
    ipos=imin;
    appendText(s);
    if(lastChar() != '\n') text.append('\n');
    ipos=imin;
  }

  /** fixe l'indice de la dernière ligne visible
      <br><i>set the index of the last visible line</i>
   @param line l'indice de la dernière ligne (-1 rend toutes les lignes visibles)
                <br><i>the index of the last visible row (-1 means that all the lines are visible)</i>
   @param column l'indice de la colonne visible (-1 rend toutes les lignes visibles)
                <br><i>the index of the last visible column (-1 means that all the lines are visible)</i> */

  public void setLastVisible(int line,int column){
   lastvisibleline=line;
   lastvisiblecolumn=column;
  }

  /** initlaise le découpeur de mots <br><i>initialize the line tokenizer</i> */

  public void initLine(){
    iroot=0;
  }

  /** get the next line */
  public String getLine(){
    int l=text.length();
    if(iroot >= l) return null;
    int ideb=iroot;
    int ifin;
    for(ifin=iroot; ifin<l;ifin++) if(text.charAt(ifin) == '\n') break;
    iroot=ifin+1;
    char a[]=new char[ifin-ideb];
    text.getChars(ideb,ifin,a,0);
    return new String(a);
  }

  /** get the line correspondng to an index */
  public String getLine(int num){
    int i=0;
    String line,delim;
    StringTokenizer st=new StringTokenizer(text.toString(),"\n",true);
    while(st.hasMoreTokens()){
      i++;
      line =st.nextToken();
      if(! line.equals("\n")) delim=st.nextToken();
      if(i == num) return line;
    }
    return null;
  }

  /** nombre de lignes du texte <br><i>return the number of lines in the text</i> */
  public int getLineNumber(){
    int n=1;
    for(int i=0; i< text.length();i++)
       if(text.charAt(i) == '\n') n++;
    return n;
  }

  /** mise à jour de la zone <br><i>update : avoiding to clear sometimes</i> */
  public void update(Graphics g){
    if(firstline != firstdraw || nline > getLineNumber()){
      firstdraw=firstline;
      nline=getLineNumber();
      g.clearRect(0,0,width,height);
    }
    paint(g);
  }

  /** dessine le texte <br><i>write the text</i> */
  public void paint(Graphics g){

    // get the true dimension

    g.setColor(Color.black);
    g.draw3DRect(0,0,width,height,true);
    if(vertical != null) vertical.setValues(firstline,nrow,1,getLineNumber());
    
    String line;
    int i;int l;
    
    g.setFont(rm);
    FontMetrics fm=g.getFontMetrics();
    g.setColor(Color.black);

    initLine();
    i=0;
    while(true){
      i++;
      line=getLine();
      if(line==null) break;
      l=i-firstline+1;
      if(l > nrow || (lastvisibleline >=0 && i > lastvisibleline)) break;
      StringTokenizer st=new StringTokenizer(line," ");
      ipos=imin;
      while(st.hasMoreTokens()){
        String s=st.nextToken();
        if(s.charAt(0) == '&'){
          if     (s.equals("&rm")){
            g.setFont(rm);
            fm=g.getFontMetrics();}
          else if(s.equals("&it")){
            g.setFont(it);
            fm=g.getFontMetrics();}
          else if(s.equals("&bf")){
            g.setFont(bf);
            fm=g.getFontMetrics();}
          else if(s.equals("&red"))
            g.setColor(Color.red);
          else if(s.equals("&green"))
            g.setColor(Color.green);
          else if(s.equals("&blue"))
            g.setColor(Color.blue);
          else if(s.equals("&black"))
            g.setColor(Color.black);
          else if(i>= firstline){
                 if(s.equals("&b" )) ipos=ipos+charwidth;
            else if(s.equals("&0" )) ipos=imin;
            else if(s.equals("&1" )) ipos=imin+charwidth;
            else if(s.equals("&2" )) ipos=imin+charwidth*2;
            else if(s.equals("&3" )) ipos=imin+charwidth*3;
            else if(s.equals("&4" )) ipos=imin+charwidth*4;
            else if(s.equals("&5" )) ipos=imin+charwidth*5;
            else if(s.equals("&6" )) ipos=imin+charwidth*6;
            else if(s.equals("&7" )) ipos=imin+charwidth*7;
            else if(s.equals("&8" )) ipos=imin+charwidth*8;
            else if(s.equals("&9" )) ipos=imin+charwidth*9;
            else if(s.equals("&10")) ipos=imin+charwidth*10;
            else if(s.equals("&11")) ipos=imin+charwidth*11;
            else if(s.equals("&12")) ipos=imin+charwidth*12;
            else if(s.equals("&13")) ipos=imin+charwidth*13;
            else if(s.equals("&14")) ipos=imin+charwidth*14;
            else if(s.equals("&15")) ipos=imin+charwidth*15;
          }
        }
        else if(i >= firstline){
          if(i==lastvisibleline)
            if(lastvisiblecolumn >= 0 && ipos >= lastvisiblecolumn) break;
          drawWord(g,s,fontsize*l);
        }
      }
    }
    nline=getLineNumber();
  }

  /** écrit un mot <br><i>write a word</i>*/
  public void drawWord(Graphics g,String s,int y){
    char[] a=s.toCharArray();
    boolean b=false;
    int z=y-fm.getAscent();
    Color old=g.getColor();
    for(int i=0; i<a.length;i++){
      if(a[i]=='^'){
        g.setColor(Color.red);
        b=true;
      }
      else {
        if(b) switch(a[i]){
          case 'b' : 
            ipos=ipos+fm.charWidth(' ');
            break;
          default :
            break;
        }
        else{
          g.drawChars(a,i,1,ipos,y);
          ipos=ipos+fm.charWidth(a[i]);
        }
        b=false;
      }
    }
    ipos=ipos+fm.charWidth(' ');
    g.setColor(old);
  }

  /** défilement ordonné par l'utilisateur <br><i>scrolling imposed by the user</i> */
  public void adjustmentValueChanged(AdjustmentEvent e){
    setRow(e.getValue());
  }

  /** ligne sélectionnée par l'utilisateur <br><i>line selected by the user</i> */
  public int selectLine(int x,int y){
    return firstline+y/fontsize;
  }

  /** défilement par utilisation des touches de curseur <br>
      <br><i>scrolling through keys pressed by the user</i>*/
  public void keyPressed(KeyEvent e){
     switch(e.getKeyCode()){
       case KeyEvent.VK_UP :
          if(firstline <= 1) return;
          firstline--;
          break;
       case KeyEvent.VK_DOWN :
          if(firstline >= getLineNumber()) return;
          firstline++;
          break;
       case KeyEvent.VK_PAGE_DOWN :
          if(firstline >= getLineNumber()) return;
          firstline=Math.min(getLineNumber()-1,firstline+nrow);
          break;
       case KeyEvent.VK_PAGE_UP :
          if(firstline <= 1) return;
          firstline=Math.max(1,firstline-nrow);
          break;
       case KeyEvent.VK_HOME :
          if(firstline <= 1) return;
          firstline=1;
          break;
       case KeyEvent.VK_END :
          if(firstline >= getLineNumber()-1) return;
          firstline=getLineNumber()-1;
          break;
       default :
          return;
     }
     if(vertical != null) 
     vertical.setValues(firstline,nrow,1,getLineNumber());
     repaint();
  }

  public void keyReleased(KeyEvent e){}
  public void keyTyped(KeyEvent e){}
  public void showLastLine(){
    int n=getLineNumber();
    if(n > firstline+nrow || n < firstline) setRow(getLineNumber()-nrow/2);       
  }
  public void mouseClicked(MouseEvent e){}
  public void mouseEntered(MouseEvent e){requestFocus();}
  public void mouseExited (MouseEvent e){}
  public void mouseReleased(MouseEvent e){}
  public void mousePressed(MouseEvent e){
    String s;
    int l;
    if(ml != null){
      int x=e.getX();
      int y=e.getY();
      l=selectLine(x,y);
      String line=getLine(l);
      if(line != null){
        if(line.startsWith("&bf &blue ")){
          StringTokenizer st=new StringTokenizer(line," .");
          s=st.nextToken();
          s=st.nextToken();
          s=st.nextToken();
          l=Integer.parseInt(s);
          ml.moveSelected(l);
        }
      }
    }
  }
  
  public void showLine(int k){
    if(k >= firstline+nrow && k < firstline) setRow(k-nrow/2);
  }
  public void addMoveListener(MoveListener ml){
    this.ml=ml;
  }
}

