package sudoku;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.swing.*;

/** la classe qui gère l'interactivité d'une zone rectangulaire pouvant contenir des candidats
  * <br><i>the class managing user interactions occuring in a rectangular area containing candidates </i>*/
public class Zone extends Canvas implements MouseListener,MouseMotionListener,ActionListener,KeyListener{

   /** liste des candidats présents dans la zone
     * <br><i> list of candidates within the zone </i> */
   public Vector list;
   /** drapeau de sélection de la zone
     * <br><i>selection flag </i> */
   public int    selection=0;
   /** taille de base de la zone
     * <br><i>basic size of the zone</i> */
   public int    size;
   /** police normle associée à cette zone (utilisée pour écrire la symbol du candidat final)
     * <br><i>normal font assocated to the zone</i> */
   public Font   fontplain;
   /** police grasse associée à cette zone
     * <br><i>bold font assocated to the zone</i> */
   public Font   fontbold;
   /** petite police associée à cette zone quand plusieurs candidats sont présents
     * <br><i>small font assocated to the zone and used when several candidates are present</i> */
   public Font   smallfontplain;
   /** petite police grasse associée à cette zone quand plusieurs candidats sont présents
     * <br><i>small bold font assocated to the zone and used when several candidates are present</i> */
   public Font   smallfontbold;
   /** couleur de fond par défaut
     * <br><i>default background color</i> */
   public static Color back=new Color(210,210,210);
   /** couleur de sélection par défaut
     * <br><i>default selection color</i> */
   public static Color backsel=Color.PINK;
   /** couleur de fond quand le pointeur de la souris survole la zone 
     * <br><i>background color when the mouse pointer is over the zone</i> */
   public static Color backmouse=new Color(230,230,230);
   /** couleur de fond  
     * <br><i>background color</i> */
   public Color background;
   /** pop menu gérant toutes les options offertes  
     * <br><i>popmenu</i> */
   public PopupMenu menu;
   /** le menu de chargement ou de sauvegarde d'un fichier  
     * <br><i>the menu to load or save a file</i> */
   public static Dialog dialog;
   static int x,y; // popmenu position within the current component
   static Color color; // current selected color
   
   public Zone(int w,int h){
      setBackground(back);
      background=back;
      list=new Vector();
      setSize(w,h);
      addMouseMotionListener(this);
      addKeyListener(this);
      addMouseListener(this);
   }
   
   public Color getCandidatColor(){
     return Color.RED.brighter();
   }
   
   /** fixer la taille d'une zone  
     * <br><i>fixing the zone size</i> */
   public void setSize(int w,int h){
      super.setSize(w,h);
      int hfont;
      if(h > w) {
        size=h; hfont=w;
      }else if(h < w) {
        size=w; hfont=h;
      }else{
        size=w; hfont=8*w/10;
        if(hfont < 15) hfont=w;
      }
      fontplain=new Font("Helvetica",Font.PLAIN,hfont);
      fontbold =new Font("Helvetica",Font.BOLD ,hfont);
      if(w != h){
         smallfontplain=fontplain;
         smallfontbold=fontbold;
      }else{
         smallfontplain=new Font("Helvetica",Font.PLAIN,hfont*2/3);
         smallfontbold =new Font("Helvetica" ,Font.BOLD,hfont*2/3);
      }
      repaint();
   }      

   /** selectionnner la zone  
     * <br><i>selecting the zone</i> */
   public void select(){
      selection=1-selection;
      if(selection == 1) setBackground(backsel); else setBackground(background);
      repaint();
   }

   /** deselectionnner la zone  
     * <br><i>unselecting the zone</i> */
   public void unselect(){
      if(selection == 0) return;
      selection=0;
      setBackground(background);
      repaint();
   }
   
   /** ajouter un candidat correspondant à un symbole particulier 
     * <br><i>adding a candidate corresponding to a particular symbol</i> */
   public void add(String value){
      int k;
      for(k=0;k< list.size();k++){
         if(get(k).value.equals(value)) return;
      }
      Candidat c=new Candidat(value);c.setColor(getCandidatColor());
      list.add(c);
      repaint();
   }
      
   /** ajouter un candidat 
     * <br><i>adding a candidate</i> */
   public void add(Candidat c){
      int k;
      for(k=0;k< list.size();k++){
         if(get(k).value.equals(c.value)) return;
      }
      list.add(c);
      repaint();
   }
   /** supprimer tous les candidats de la zone 
     * <br><i>deleting all candidates</i> */
   public void clear(){list.clear();unselect();}
   /** récupérer un candidat par son numéro
     * <br><i>getting a candidate by its index</i> */
   public Candidat get(int i){try{return (Candidat)list.get(i);}catch(Exception e){return null;}}
   /** récupérer un candidat par son nom
     * <br><i>getting a candidate by its name</i> */
   public Candidat get(String name){
      for(int i=0;i < list.size(); i++){
         Candidat c=(Candidat)list.get(i);
         if(c.value.equals(name)) return c;
      }
      return null;
   }
   /** supprimer le candidat à la position indiquée
     * <br><i>deleting a candidate at the given index</i> */
   public void delete(int i){try{list.removeElementAt(i);}catch(Exception e){}}
   /** supprimer le candidat dont le symbol correspond à celui du candidat passé en argument 
     * <br><i>deleting a candidate which symbol corresponds to the candidat passed as input argument</i> */
   public void delete(Candidat c){
      int k;
      for(k=0;k< list.size();k++){
         if(get(k).value.equals(c.value)) delete(k);
      }
   }
   /** supprimer le candidat dont le symbol correspond à celui passé par argument
     * <br><i>deleting a candidate which symbol corresponds to those passed by argument</i> */
   public void delete(String value){
      int k;
      for(k=0;k< list.size();k++){
         if(get(k).value.equals(value)) delete(k);
      }
   }
   
   /** construire l'image correspondant à la zone
     * <br><i>building up the image corresponding to the zone</i> */
   public Image getImage(){
       Image img=createImage(getWidth(),getHeight());
       paint(img.getGraphics());
       return img;
   }
   
   /** tracer la zone
     * <br><i>drawing the zone</i> */
   public void paint(Graphics g){
     drawCandidat(g);
   }
   
   /** écrire les candidats dans la zone
     * <br><i>writing candidates</i> */
   public void drawCandidat(Graphics g){
      int i,j,k,w,h;
      Candidat c;
      if(list.size() == 0) return;
      if(list.size() == 1)
         g.setFont(fontbold);
      else
         g.setFont(smallfontbold);
      for(k=0;k<list.size();k++){
         c=(Candidat)list.get(k);
         w=getWidth();
         h=getHeight();
         if(w > h){
            i=w/list.size()/2+k*w/list.size();
            j=h/2+c.position*h/7;
         }else if(w < h){
            j=h/list.size()/2+k*h/list.size();
            i=w/2+c.position*w/5;
         }else{
            if(list.size() == 1) {
               i=w/2;
               j=w/2;
            }else if(list.size() == 2) {
               if(k % 2 == 0) i=w/4 ; else i=3*w/4;
               j=w/2;
            }else if(list.size() == 3) {
               if(k % 3 == 0) i=w/6 ; else if (k%3 == 1) i=w/2; else i=5*w/6;
               j=w/2;
            }else if(list.size() == 4) {
               if(k % 2 == 0) i=w/4 ; else i=3*w/4;
               if(k < 2) j=h/4 ; else j=3*h/4;
            }else if(list.size() <= 6){
               if(k % 3 == 0) i=w/6 ; else if (k%3 == 1) i=w/2; else i=5*w/6;
               if(k < 3) j=h/4 ; else j=3*h/4;
            }else{
               if(k % 3 == 0) i=w/6 ; else if (k%3 == 1) i=w/2; else i=5*w/6;
               if(k < 3) j=h/6 ; else if (k < 6) j=h/2; else j=5*h/6;
            }
         }
         c.paint(g,i,j);
      }
   }
   
   /** gestion de l'entrée du pointeur
     * <br><i>handling input focus</i> */
   public void mouseEntered (MouseEvent e){
      setBackground(backmouse);
      repaint();
      requestFocus();
   }

   /** gestion de la sortie du pointeur
     * <br><i>handling output focus</i> */
   public void mouseExited  (MouseEvent e){
      if(selection == 1) setBackground(backsel);else setBackground(background);
      repaint();
   }
   
   /** gestion d'un déplacement du pointeur dans la zone
     * <br><i>handling a mouse move within the zone</i> */
   public void mouseMoved(MouseEvent e){
     x=e.getX();
     y=e.getY();
     requestFocus();
   }

   /** gestion d'un clic de souris
     * <br><i>handling a mouse click</i> */
   public void mousePressed (MouseEvent e){
      MenuItem item;
      Menu submenu;
      int button=e.getButton();
      x=e.getX();
      y=e.getY();
      if(button >= 2) {
         if(menu == null) {
            menu = new PopupMenu();
            submenu = new Menu(RF.get("menu.modify_candidate"));menu.add(submenu);
            item = new MenuItem(RF.get("menu.delete_selected_candidate"));item.addActionListener(this);submenu.add(item);
            item = new MenuItem(RF.get("menu.stripe_selected_candidate"));item.addActionListener(this);submenu.add(item);
            item = new MenuItem(RF.get("menu.delete_zone_candidates"));item.addActionListener(this);submenu.add(item);
            item = new MenuItem(RF.get("menu.delete_all_candidates"));item.addActionListener(this);submenu.add(item);
            item = new MenuItem(RF.get("menu.keep_only_black_candidates"));item.addActionListener(this);submenu.add(item);
            submenu = new Menu(RF.get("menu.add_new_candidate"));menu.add(submenu);
            item=new MenuItem("1");item.addActionListener(this);submenu.add(item);
            item=new MenuItem("2");item.addActionListener(this);submenu.add(item);
            item=new MenuItem("3");item.addActionListener(this);submenu.add(item);
            item=new MenuItem("4");item.addActionListener(this);submenu.add(item);
            item=new MenuItem("5");item.addActionListener(this);submenu.add(item);
            item=new MenuItem("6");item.addActionListener(this);submenu.add(item);
            item=new MenuItem("7");item.addActionListener(this);submenu.add(item);
            item=new MenuItem("8");item.addActionListener(this);submenu.add(item);
            item=new MenuItem("9");item.addActionListener(this);submenu.add(item);
            submenu = new Menu(RF.get("menu.help"));menu.add(submenu);
            item = new MenuItem(RF.get("menu.compute_candidates"));item.addActionListener(this);submenu.add(item);
            item = new MenuItem(RF.get("menu.solve_position"));item.addActionListener(this);submenu.add(item);
            submenu = new Menu(RF.get("menu.move_candidate"));menu.add(submenu);
            item=new MenuItem(RF.get("menu.left"));item.addActionListener(this);submenu.add(item);
            item=new MenuItem(RF.get("menu.right"));item.addActionListener(this);submenu.add(item);
            item=new MenuItem(RF.get("menu.up"));item.addActionListener(this);submenu.add(item);
            item=new MenuItem(RF.get("menu.down"));item.addActionListener(this);submenu.add(item);
            submenu = new Menu(RF.get("menu.change_color"));menu.add(submenu);
            item=new MenuItem(RF.get("menu.black"));item.addActionListener(this);submenu.add(item);
            item=new MenuItem(RF.get("menu.red"))  ;item.addActionListener(this);submenu.add(item);
            item=new MenuItem(RF.get("menu.green"));item.addActionListener(this);submenu.add(item);
            item=new MenuItem(RF.get("menu.blue")) ;item.addActionListener(this);submenu.add(item);
            submenu = new Menu(RF.get("menu.select"));menu.add(submenu);
            item = new MenuItem(RF.get("menu.select_row"));item.addActionListener(this);submenu.add(item);
            item = new MenuItem(RF.get("menu.select_column"));item.addActionListener(this);submenu.add(item);
            item = new MenuItem(RF.get("menu.select_section"));item.addActionListener(this);submenu.add(item);
            item = new MenuItem(RF.get("menu.select_zone"));item.addActionListener(this);submenu.add(item);
            submenu = new Menu(RF.get("menu.color"));menu.add(submenu);
            item = new MenuItem(RF.get("menu.backcolor"));item.addActionListener(this);submenu.add(item);
            item = new MenuItem(RF.get("menu.backselcolor"));item.addActionListener(this);submenu.add(item);
            item = new MenuItem(RF.get("menu.backmousecolor"));item.addActionListener(this);submenu.add(item);
            submenu = new Menu(RF.get("menu.save"));menu.add(submenu);
            item = new MenuItem(RF.get("menu.save_view"));item.addActionListener(this);submenu.add(item);
            item = new MenuItem(RF.get("menu.print_position"));item.addActionListener(this);submenu.add(item);
            item = new MenuItem(RF.get("menu.save_position"));item.addActionListener(this);submenu.add(item);
            add(menu);
         }
         menu.show(this,x,y);
      }
      requestFocus();
   }
   
   /** gestion des actions déclenchées par le popmenu
     * <br><i>handling the actions associated to the popmenu</i> */
   public void actionPerformed(ActionEvent e){
      String label=((MenuItem)e.getSource()).getLabel();
      int k=getIndex(x,y);
      Candidat c=null;
      if(k>=0) c=(Candidat)list.get(k);
      if(label.equals(RF.get("menu.delete_zone_candidates"))){
         clear();repaint();
         Move.record(Move.getZoneName(this)+'d');
      }else if(label.equals(RF.get("menu.delete_all_candidates"))){
         Board b=(Board)getParent();
         b.position.emptyPosition();
         b.repaintAll();
         Move.record(Move.getZoneName(this)+'D');
      }else if(label.equals(RF.get("menu.keep_only_black_candidates"))){
         Board b=(Board)getParent();
         for(int j=0;j<b.getComponentCount();j++){
           Zone z=(Zone)b.getComponent(j);
           for(int i=0; i < z.list.size(); i++) if(z.get(i).color != Color.BLACK) {z.delete(k);z.repaint();};
         }
         Move.record(Move.getZoneName(this)+'N');
      }else if(label.equals(RF.get("menu.stripe_selected_candidate"))){
         if(k >= 0 ) {c.changeBarre();repaint();}         
         Move.record(Move.getZoneName(this)+'x'+c.value);
      }else if(label.equals(RF.get("menu.save_view"))){
         Container parent=getParent();
         while(! (parent instanceof Frame)) parent=parent.getParent();
         FileDialog fd=new FileDialog((Frame)parent,RF.get("dialog.png_save"),FileDialog.SAVE);
         fd.show();
         String filename=fd.getFile();
         if(filename != null) {
           File file=new File(fd.getDirectory(),fd.getFile());
           Board b=(Board)getParent();
           b.saveImage(file);
         }
      }else if(label.equals(RF.get("menu.print_position"))){
         Board b=(Board)getParent();
         Position.printFull(b.position.cells);
      }else if(label.equals(RF.get("menu.save_position"))){
         Board b=(Board)getParent();
         System.out.println(b.position.saveFullPosition());
      }else if(label.equals(RF.get("menu.solve_position"))){
         Board b=(Board)getParent();
         int nsol=Algo.fastSolver(b.position.cells,0);
         System.out.println("number of solutions : "+nsol);
         for(int i=0;i<b.n;i++)for(int j=0;j<b.n;j++)b.position.cells[i][j].repaint();
         Move.record(Move.getZoneName(this)+'X');
      }else if(label.equals(RF.get("menu.delete_selected_candidate"))){
         if(k >= 0) {
           delete(k);repaint();
           Move.record(Move.getZoneName(this)+'s'+c.value);
         }
      }else if(label.equals(RF.get("menu.compute_candidates"))){
         if(this instanceof Cell){
            if(this.list.size() != 1) {
               Board b=(Board)getParent();
               b.position.fillCandidats((Cell)this,b.position.cells);
               repaint();
               Move.record(Move.getZoneName(this)+'c');
            }
         }
      }else if(label.equals(RF.get("menu.left"))){
         if(this instanceof Junction && getWidth() < getHeight() && k >= 0){
            c.setPosition(c.getPosition()-1);
            repaint();
            Move.record(Move.getZoneName(this)+'-'+c.value);
         }
      }else if(label.equals(RF.get("menu.right"))){
         if(this instanceof Junction && getWidth() < getHeight() && k >= 0){
            c.setPosition(c.getPosition()+1);
            repaint();
            Move.record(Move.getZoneName(this)+'+'+c.value);
         }
      }else if(label.equals(RF.get("menu.up"))){
         if(this instanceof Junction && getWidth() > getHeight() && k >= 0){
            c.setPosition(c.getPosition()-1);
            repaint();
            Move.record(Move.getZoneName(this)+'+'+c.value);
         }
      }else if(label.equals(RF.get("menu.down"))){
         if(this instanceof Junction && getWidth() > getHeight() && k >= 0){
            c.setPosition(c.getPosition()+1);
            repaint();
            Move.record(Move.getZoneName(this)+'-'+c.value);
         }
      }else if(label.equals("1")){
         add(label);repaint();
         Move.record(Move.getZoneName(this)+"=1");
      }else if(label.equals("2")){
         add(label);repaint();
         Move.record(Move.getZoneName(this)+"=2");
      }else if(label.equals("3")){
         add(label);repaint();
         Move.record(Move.getZoneName(this)+"=3");
      }else if(label.equals("4")){
         add(label);repaint();
         Move.record(Move.getZoneName(this)+"=4");
      }else if(label.equals("5")){
         add(label);repaint();
         Move.record(Move.getZoneName(this)+"=5");
      }else if(label.equals("6")){
         add(label);repaint();
         Move.record(Move.getZoneName(this)+"=6");
      }else if(label.equals("7")){
         add(label);repaint();
         Move.record(Move.getZoneName(this)+"=7");
      }else if(label.equals("8")){
         add(label);repaint();
         Move.record(Move.getZoneName(this)+"=8");
      }else if(label.equals("9")){
         add(label);repaint();
         Move.record(Move.getZoneName(this)+"=9");
      }else if(label.equals(RF.get("menu.red"))){
         color=Color.RED;
         if(k >= 0) {
            c.setColor(color);
            repaint();
            Move.record(Move.getZoneName(this)+'r'+c.value);
         }
      }else if(label.equals(RF.get("menu.green"))){
         color=Color.GREEN;
         if(k >= 0) {
            c.setColor(color);
            repaint();
            Move.record(Move.getZoneName(this)+'g'+c.value);
         }
      }else if(label.equals(RF.get("menu.blue"))){
         color=Color.BLUE;
         if(k >= 0) {
            c.setColor(color);
            repaint();
            Move.record(Move.getZoneName(this)+'b'+c.value);
         }
      }else if(label.equals(RF.get("menu.black"))){
         color=Color.BLACK;
         if(k >= 0) {
            c.setColor(color);
            repaint();
            Move.record(Move.getZoneName(this)+'n'+c.value);
         }
      }else if(label.equals(RF.get("menu.select_zone"))){
         Board b=(Board)getParent();
         b.selectZone(this);
         Move.record(Move.getZoneName(this)+'Z');
      }else if(label.equals(RF.get("menu.select_row"))){
         Board b=(Board)getParent();
         b.selectRow(this);
         Move.record(Move.getZoneName(this)+'R');
      }else if(label.equals(RF.get("menu.select_column"))){
         Board b=(Board)getParent();
         b.selectColumn(this);
         Move.record(Move.getZoneName(this)+'C');
      }else if(label.equals(RF.get("menu.select_section"))){
         Board b=(Board)getParent();
         b.selectSection(this);
         Move.record(Move.getZoneName(this)+'S');
      }else if(label.equals(RF.get("menu.backcolor"))){
         try{
           Color color=JColorChooser.showDialog(this,RF.get("menu.backcolor"),back);
           if(!color.equals(back)){
              Board b=(Board)getParent();
              b.changeBackground(back,color);
              b.repaintAll();
              back=color;
           }
         }catch(Exception ex){System.out.println(ex.toString());}
      }else if(label.equals(RF.get("menu.backselcolor"))){
         try{
           backsel=JColorChooser.showDialog(this,RF.get("menu.backselcolor"),backsel);
         }catch(Exception ex){System.out.println(ex.toString());}
      }else if(label.equals(RF.get("menu.backmousecolor"))){
         try{
           backmouse=JColorChooser.showDialog(this,RF.get("menu.backmousecolor"),backmouse);
           repaint();
         }catch(Exception ex){System.out.println(ex.toString());}
      }
     requestFocus();
   }
   
   /** numéro du candidat situé aux coordonnées fournies
     * <br><i>index of the candidate at given coordinates</i> */
   public int getIndex(int x,int y){
      int i,j,k;
      k=-1;
      if(list.size() == 0) return k;
      if(list.size() <= 1){
         k=0;
         return k;
      }
      int w=getWidth();
      int h=getHeight();
      if(w > h){
         k=x*list.size()/w;
      }else if(w < h){ 
         k=y*list.size()/h;
      }else if(list.size() == 2){
         if(x < w/2) k=0; else k=1;
      }else if(list.size() == 3){
         if(x < w/3) k=0; else if(x < 2*w/3) k=1; else k=2;
      }else if(list.size() == 4){
         if(x < w/2) i=0; else i=1;
         if(y < h/2) j=0; else j=1;
         k=2*j+i;
      }else if(list.size() <= 6){
         if(x < w/3) i=0; else if(x < 2*w/3) i=1;else  i=2;
         if(y < h/2) j=0; else  j=1;
         k=3*j+i;
      }else if(list.size() <= 9){
         if(x < w/3) i=0; else if(x < 2*w/3) i=1;else  i=2;
         if(y < h/3) j=0; else if(y < 2*h/3) j=1;else  j=2;
         k=3*j+i;
      }
      if(k < 0 || k >= list.size()) k=-1;
      return k;
   }

   /** numéro du candidat dont on connait le symbole
     * <br><i>index of the candidate with a given symbol</i> */
   public int getIndex(String value){
      int k;
      for(k=0;k<list.size();k++){
         Candidat c=(Candidat)list.get(k);
         if(c.value.equals(value)) return k;
      }
      return -1;
   }

   /** numéro du candidat dont le symbole correspond à celui du candidat passé en argument
     * <br><i>index of the candidate which symbol corresponds to those of the candidate passed by argument</i> */
   public int getIndex(Candidat c){
      int k;
      for(k=0;k<list.size();k++){
         Candidat cand=(Candidat)list.get(k);
         if(cand.value.equals(c.value)) return k;
      }
      return -1;
   }
   
   /** gestion des racourcis "clavier" du pop-menu
     * <br><i>managing keyboard shortcuts associated to the popmenu</i> */
   public void keyPressed(KeyEvent e){
      int k=getIndex(x,y);
      Candidat c=null;
      if(k >= 0) c=(Candidat)list.get(k);
      char ckey=e.getKeyChar();
      int key=e.getKeyCode();
      if(key == 127 || key == 8){
         if(k >= 0) {
           delete(k);repaint();
           Move.record(Move.getZoneName(this)+'s'+c.value);
         }
      }else if(ckey == KeyEvent.CHAR_UNDEFINED){
         if(key == 37){
            if(this instanceof Junction && getWidth() < getHeight() && k >= 0){
               c.setPosition(c.getPosition()-1);
               repaint();
               Move.record(Move.getZoneName(this)+'-'+c.value);
            }
         }else if(key == 38){
            if(this instanceof Junction && getWidth() > getHeight() && k >= 0){
               c.setPosition(c.getPosition()-1);
               repaint();
               Move.record(Move.getZoneName(this)+'-'+c.value);
            }
         }else if(key == 39){
            if(this instanceof Junction && getWidth() < getHeight() && k >= 0){
               c.setPosition(c.getPosition()+1);
               repaint();
               Move.record(Move.getZoneName(this)+'+'+c.value);
            }
         }else if(key == 40){
            if(this instanceof Junction && getWidth() > getHeight() && k >= 0){
               c.setPosition(c.getPosition()+1);
               repaint();
               Move.record(Move.getZoneName(this)+'+'+c.value);
            }
         }
      }else{         
         if(Character.isDigit(ckey)){
            if(ckey != '0' && list.size() < 9) {
               String s=String.valueOf(ckey);
               add(s);
               repaint();
               Move.record(Move.getZoneName(this)+'='+s);
            }
         }else if(ckey == 'x' && k>=0){
            c.changeBarre();
            repaint();
            Move.record(Move.getZoneName(this)+'x'+c.value);
         }else if(ckey == 'c'){
            if(this instanceof Cell){
               if(this.list.size() != 1) {
                  Board b=(Board)getParent();
                  b.position.fillCandidats((Cell)this,b.position.cells);
                  repaint();
                  Move.record(Move.getZoneName(this)+'c');
               }
            }
         }else if(ckey == 'd'){
            clear();repaint();
            Move.record(Move.getZoneName(this)+'d');
         }else if(ckey == 'D'){
            Board b=(Board)getParent();
            b.position.emptyPosition();
            for(int j=0;j<b.getComponentCount();j++) b.getComponent(j).repaint();
            Move.record(Move.getZoneName(this)+'D');
         }else if(ckey == 'K'){
            Board b=(Board)getParent();
            for(int j=0;j<b.getComponentCount();j++){
               Zone z=(Zone)b.getComponent(j);
               for(int i=0; i < z.list.size(); i++) if(z.get(i).color != Color.BLACK) {z.delete(k);z.repaint();}
            }
            Move.record(Move.getZoneName(this)+'K');
         }else if(ckey == 'X'){
            Board b=(Board)getParent();
            int nsol=Algo.fastSolver(b.position.cells,0);
            System.out.println("number of solutions : "+nsol);
            for(int i=0;i<b.n;i++)for(int j=0;j<b.n;j++)b.position.cells[i][j].repaint();
            Move.record(Move.getZoneName(this)+'X');
         }else if(ckey == 'p'){
            Board b=(Board)getParent();
            Position.printFull(b.position.cells);
         }else if(ckey == 's'){
            Board b=(Board)getParent();
            System.out.println(b.position.saveFullPosition());
         }else if(ckey == 'v'){
            Container parent=getParent();
            while(! (parent instanceof Frame)) parent=parent.getParent();
            FileDialog fd=new FileDialog((Frame)parent,RF.get("dialog.png_save"),FileDialog.SAVE);
            fd.show();
            String filename=fd.getFile();
            if(filename != null){
              Board b=(Board)getParent();
              b.saveImage(new File(fd.getDirectory(),filename));
            }
         }else if(ckey == 'R' || ckey == 'L'){
            Board b=(Board)getParent();
            b.selectRow(this);
            Move.record(Move.getZoneName(this)+'R');
         }else if(ckey == 'C'){
            Board b=(Board)getParent();
            b.selectColumn(this);
            Move.record(Move.getZoneName(this)+'C');
         }else if(ckey == 'S'){
            Board b=(Board)getParent();
            b.selectSection(this);
            Move.record(Move.getZoneName(this)+'S');
         }else if(ckey == 'Z'){
            Board b=(Board)getParent();
            b.selectZone(this);
            Move.record(Move.getZoneName(this)+'Z');
         }else if(ckey == 'r' && k>=0){
            c.setColor(Color.RED);
            repaint();
            Move.record(Move.getZoneName(this)+'r'+c.value);
         }else if(ckey == 'g' || ckey == 'v' && k>=0){
            c.setColor(Color.GREEN);
            repaint();
            Move.record(Move.getZoneName(this)+'g'+c.value);
         }else if(ckey == 'b' && k>=0){
            c.setColor(Color.BLUE);
            repaint();
            Move.record(Move.getZoneName(this)+'b'+c.value);
         }else if(ckey == 'n' && k>=0){
            c.setColor(Color.BLACK);
            repaint();
            Move.record(Move.getZoneName(this)+'n'+c.value);
         }
     }
     requestFocus();
   }
   
   public void keyReleased(KeyEvent e){}
   public void keyTyped(KeyEvent e){}
   public void mouseDragged(MouseEvent e){}
   public void mouseReleased(MouseEvent e){}
   public void mouseClicked (MouseEvent e){}

}
